use clap::Parser;
use std::sync::Arc;

#[derive(Parser)]
pub struct AppConfigInner {
    #[clap(long, env)]
    pub database_url: String,

    #[clap(long, env)]
    pub rust_log: String,

    #[clap(long, env)]
    pub external_port: u32,

    #[clap(long, env)]
    pub internal_port: u32,

    #[clap(long, env)]
    pub run_migrations: bool,

    #[clap(long, env)]
    pub cors_origin: String,
}

pub struct AppConfig {
    inner: Arc<AppConfigInner>,
}

impl AppConfig {
    pub fn new() -> Self {
        Self {
            inner: Arc::new(AppConfigInner::parse()),
        }
    }

    pub fn cfg(&self) -> Arc<AppConfigInner> {
        self.inner.clone()
    }

    pub fn rust_log(&self) -> &str {
        &self.inner.rust_log
    }

    pub fn cors_origin(&self) -> &str {
        &self.inner.rust_log
    }

    pub fn external_port(&self) -> &u32 {
        &self.inner.external_port
    }

    pub fn internal_port(&self) -> &u32 {
        &self.inner.internal_port
    }
}
